pub struct ForwardADMulti<A: AD + ForwardADTrait> { /* private fields */ }Expand description
Computes derivatives using Forward-mode Automatic Differentiation (Multi-Tangent).
This method allows for computing multiple columns of the Jacobian in a single pass by propagating a vector of tangents. This can significantly speed up computation by taking advantage of SIMD and reducing function overhead.
Implementations
Sourceimpl<A: AD + ForwardADTrait> ForwardADMulti<A>
impl<A: AD + ForwardADTrait> ForwardADMulti<A>
Sourcepub fn new() -> Self
pub fn new() -> Self
Examples found in repository?
examples/example1.rs (line 63)
33fn main() {
34 let inputs = vec![1., 2.];
35
36 // Reverse AD //////////////////////////////////////////////////////////////////////////////////
37 let function_standard = Test { coeff: 2.0 };
38 let function_derivative = function_standard.to_other_ad_type::<adr>();
39 let differentiable_block = FunctionEngine::new(function_standard, function_derivative, ReverseAD::new());
40
41 let (f_res, derivative_res) = differentiable_block.derivative(&inputs);
42 println!("Reverse AD: ");
43 println!(" f_res: {}", f_res[0]);
44 println!(" derivative: {}", derivative_res);
45 println!("//////////////");
46 println!();
47
48 // Forward AD //////////////////////////////////////////////////////////////////////////////////
49 let function_standard = Test { coeff: 2.0 };
50 let function_derivative = function_standard.to_other_ad_type::<adfn<1>>();
51 let differentiable_block = FunctionEngine::new(function_standard, function_derivative, ForwardAD::new());
52
53 let (f_res, derivative_res) = differentiable_block.derivative(&inputs);
54 println!("Forward AD: ");
55 println!(" f_res: {}", f_res[0]);
56 println!(" derivative: {}", derivative_res);
57 println!("//////////////");
58 println!();
59
60 // Forward AD Multi ////////////////////////////////////////////////////////////////////////////
61 let function_standard = Test { coeff: 2.0 };
62 let function_derivative = function_standard.to_other_ad_type::<adfn<2>>();
63 let differentiable_block = FunctionEngine::new(function_standard, function_derivative, ForwardADMulti::new());
64
65 let (f_res, derivative_res) = differentiable_block.derivative(&inputs);
66 println!("Forward AD Multi: ");
67 println!(" f_res: {}", f_res[0]);
68 println!(" derivative: {}", derivative_res);
69 println!("//////////////");
70 println!();
71
72 // Finite Differencing /////////////////////////////////////////////////////////////////////////
73 let function_standard = Test { coeff: 2.0 };
74 let function_derivative = function_standard.clone();
75 let differentiable_block = FunctionEngine::new(function_standard, function_derivative, FiniteDifferencing::new());
76
77 let (f_res, derivative_res) = differentiable_block.derivative(&inputs);
78 println!("Finite Differencing: ");
79 println!(" f_res: {}", f_res[0]);
80 println!(" derivative: {}", derivative_res);
81 println!("//////////////");
82 println!();
83
84}Trait Implementations
Sourceimpl<A: Clone + AD + ForwardADTrait> Clone for ForwardADMulti<A>
impl<A: Clone + AD + ForwardADTrait> Clone for ForwardADMulti<A>
Sourcefn clone(&self) -> ForwardADMulti<A>
fn clone(&self) -> ForwardADMulti<A>
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl<A: AD + ForwardADTrait> DerivativeMethodTrait for ForwardADMulti<A>
impl<A: AD + ForwardADTrait> DerivativeMethodTrait for ForwardADMulti<A>